/* $Workfile: hpdi32_print.c $*/
/* $Revision: 1 $*/
/* $Date: 11/14/03 11:30a $*/
/*****************************************************************************
*
*	Implement a menu driver test application for the HPDI32-print card.
*	This is based on HPDI32-print.cpp, which used the NT 4 driver.
*
*****************************************************************************/

/* $Log: /NEW SOURCESAFE DATABASE/PLX SDK 4.0.0 Drivers - Don/gsc/hpdi32/Win32/print/hpdi32_print.c $
 * 
 * 1     11/14/03 11:30a Dnewbold
*/

#include <windows.h>
#include <conio.h>
#include <stdio.h>

#include "hpdi32_api.h"
#include "hpdi32_api_print.h"



/* #defines	*****************************************************************/

#define	ARRAY_ELEMENTS(a)	(sizeof((a))/sizeof((a)[0]))
#define	IGNORE_ARG(a)		{if((a)){}}

// These are specific to the test board.
#define	HPDI32_PRINT_TEST_BOARD_QTDCR		HPDI32_REG_ENCODE(4, 0x28)
#define	HPDI32_PRINT_TEST_BOARD_PCSR		HPDI32_REG_ENCODE(4, 0x2C)



/* typedefs	*****************************************************************/

typedef	enum
{
	BOARD_TYPE_OTHER,
	BOARD_TYPE_RX,
	BOARD_TYPE_TX
} board_type_t;

typedef void	(*menu_func_t)(	int		tx_index,
								void*	tx_handle,
								int		rx_index,
								void*	rx_handle);

typedef struct
{
	const char*	text;
	menu_func_t	function;
} menu_item_t;

typedef struct
{
	const char*	name;
	U32			reg;
	int			rw;		// Read/Write access?
	U32			mask;
} reg_t;



/* variables	*************************************************************/

#define	_MACRO(m)	m, #m
static const struct
{
	U32			status;
	const char*	text;
} _err_list[]	=
{
    { _MACRO(GSC_FAILED)						},
    { _MACRO(GSC_ACCESS_DENIED)					},
    { _MACRO(GSC_DMA_CHANNEL_UNAVAILABLE)		},
    { _MACRO(GSC_DMA_CHANNEL_INVALID)			},
    { _MACRO(GSC_DMA_CHANNEL_TYPE_ERROR)		},
    { _MACRO(GSC_DMA_IN_PROGRESS)				},
    { _MACRO(GSC_DMA_DONE)						},
    { _MACRO(GSC_DMA_PAUSED)					},
    { _MACRO(GSC_DMA_NOT_PAUSED)				},
    { _MACRO(GSC_DMA_CHANNEL_COMMAND_INVALID)	},
    { _MACRO(GSC_DMA_MAN_READY)					},
    { _MACRO(GSC_DMA_MAN_NOT_READY)				},
    { _MACRO(GSC_DMA_INVALID_CHANNEL_PRIORITY)	},
    { _MACRO(GSC_DMA_MAN_CORRUPTED)				},
    { _MACRO(GSC_DMA_INVALID_ELEMENT_INDEX)		},
    { _MACRO(GSC_DMA_NO_MORE_ELEMENTS)			},
    { _MACRO(GSC_DMA_SGL_INVALID)				},
    { _MACRO(GSC_DMA_SGL_QUEUE_FULL)			},
    { _MACRO(GSC_NULL_PARAM)					},
    { _MACRO(GSC_INVALID_BUS_INDEX)				},
    { _MACRO(GSC_UNSUPPORTED_FUNCTION)			},
    { _MACRO(GSC_INVALID_PCI_SPACE)				},
    { _MACRO(GSC_INVALID_IOP_SPACE)				},
    { _MACRO(GSC_INVALID_SIZE)					},
    { _MACRO(GSC_INVALID_ADDRESS)				},
    { _MACRO(GSC_INVALID_ADDRESS_TYPE)			},
    { _MACRO(GSC_INVALID_INDEX)					},
    { _MACRO(GSC_MU_NOT_READY)					},
    { _MACRO(GSC_MU_FIFO_EMPTY)					},
    { _MACRO(GSC_MU_FIFO_FULL)					},
    { _MACRO(GSC_INVALID_REGISTER)				},
    { _MACRO(GSC_DOORBELL_CLEAR_FAILED)			},
    { _MACRO(GSC_INVALID_USER_PIN)				},
    { _MACRO(GSC_INVALID_USER_STATE)			},
    { _MACRO(GSC_EEPROM_NOT_PRESENT)			},
    { _MACRO(GSC_EEPROMP_TYPE_NOT_SUPPORTED)	},
    { _MACRO(GSC_EEPROM_BLANK)					},
    { _MACRO(GSC_CONFIG_ACCESS_FAILED)			},
    { _MACRO(GSC_INVALID_DEVICE_INFO)			},
    { _MACRO(GSC_NO_ACTIVE_DRIVER)				},
    { _MACRO(GSC_INSUFFICIENT_RESOURCES)		},
    { _MACRO(GSC_OBJECT_ALREADY_ALLOCATED)		},
    { _MACRO(GSC_ALREADY_INITIALIZED)			},
    { _MACRO(GSC_NOT_INITIALIZED)				},
    { _MACRO(GSC_BAD_CONFIG_REG_ENDIAN_MODE)	},
    { _MACRO(GSC_INVALID_POWER_STATE)			},
    { _MACRO(GSC_POWER_DOWN)					},
    { _MACRO(GSC_FLYBY_NOT_SUPPORTED)			},
    { _MACRO(GSC_CHANNEL_NOT_SUPPORTED)			},
    { _MACRO(GSC_NO_ACTION)						},
    { _MACRO(GSC_HS_NOT_SUPPORTED)				},
    { _MACRO(GSC_VP_NOT_SUPPORTED)				},
    { _MACRO(GSC_VPD_NOT_ENABLED)				},
    { _MACRO(GSC_NO_MORE_CAP)					},
    { _MACRO(GSC_INVALID_OFFSET)				},
    { _MACRO(GSC_BAD_PIN_DIRECTION)				},
    { _MACRO(GSC_PCI_TIMEOUT)					},
    { _MACRO(GSC_DMA_CHANNEL_CLOSED)			},
    { _MACRO(GSC_DMA_CHANNEL_ERROR)				},
    { _MACRO(GSC_INVALID_HANDLE)				},
    { _MACRO(GSC_BUFFER_NOT_READY)				},
    { _MACRO(GSC_INVALID_DATA)					},
    { _MACRO(GSC_DO_NOTHING)					},
    { _MACRO(GSC_DMA_SGL_BUILD_FAILED)			},
    { _MACRO(GSC_PM_NOT_SUPPORTED)				},
    { _MACRO(GSC_INVALID_VERSION_DRIVER)		},
    { _MACRO(GSC_WAIT_TIMEOUT)					},
    { _MACRO(GSC_WAIT_CANCELED)					},

	{ _MACRO(GSC_LIB_LOAD_FAILURE)				},
	{ _MACRO(GSC_LIB_FUNCTION_ACCESS_FAILURE)	},
    { _MACRO(GSC_INIT_FAILURE)					},
    { _MACRO(GSC_INVALID_VERSION_API)			},
    { _MACRO(GSC_INVALID_VERSION_DIL)			},
	{ _MACRO(GSC_INTERNAL_ERROR)				},
	{ _MACRO(GSC_TOO_MANY_OPEN_HANDLES)			},
	{ _MACRO(GSC_INVALID_API_HANDLE)			},
	{ _MACRO(GSC_THREAD_FAILURE)				},
	{ _MACRO(GSC_ABORTED)						}
};



/*****************************************************************************
*
*	Function:	_format_float
*
*	Purpose:
*
*		Format a value into a string with a factor suffix.
*
*	Arguments:
*
*		value	The value to format.
*
*		dest	The destination buffer.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _format_float(float value, char* dest)
{
	#define	_1K		(1000.)
	#define	_10K	(10000.)
	#define	_100K	(100000.)
	#define	_1M		(1000000.)
	#define	_10M	(10000000.)
	#define	_100M	(100000000.)
	#define	_1G		(1000000000.)
	#define	_10G	(10000000000.)
	#define	_100G	(100000000000.)

	static const struct
	{
		double		limit;
		const char*	format;
		double		div;
	} rates[]	=
	{
		/* limit	format		div	*/
		{ _1K,		"%.0f",		1	},
		{ _10K,		"%.3fK",	_1K	},
		{ _100K,	"%.2fK",	_1K	},
		{ _1M,		"%.1fK",	_1K	},
		{ _10M,		"%.3fM",	_1M	},
		{ _100M,	"%.2fM",	_1M	},
		{ _1G,		"%.1fM",	_1M	},
		{ _10G,		"%.3fG",	_1G	},
		{ _100G,	"%.3fG",	_1G	},
		{ -1,		"%.3fG",	_1G	}
	};

	int	i;

	for (i = 0; i < (int) ARRAY_ELEMENTS(rates); i++)
	{
		if ((value < rates[i].limit) || (rates[i].limit == -1))
		{
			sprintf(dest, rates[i].format, (double) value / rates[i].div);
			break;
		}
	}
}



/*****************************************************************************
*
*	Function:	_introductions
*
*	Purpose:
*
*		Display introductory information.
*
*	Arguments:
*
*		tx_index	The intended index for the transmit board. If < 0, then
*					the board index is not displayed.
*
*		rx_index	The intended index for the receive board. If < 0, then
*					the board index is not displayed.
*
*		msg			An additional message to display. If NULL then none is
*					displayed.
*
*		escape		Text for what the ESCAPE does.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _introductions(
	int			tx_index,
	int			rx_index,
	const char*	msg,
	const char*	escape)
{
	system("cls");
	printf("\n");
	printf("  PCI-HPDI32A-PRINT BOARD TEST APPLICATION\n");
	printf("\n");

	if (tx_index >= 0)
		printf("    The PRINT board is index %d.\n", tx_index);

	if (rx_index >= 0)
		printf("    The TEST board is index %d.\n", rx_index);

	if ((tx_index >= 0) || (rx_index >= 0))
		printf("\n");

	if (msg)
		printf("  %s\n", msg);

	if (escape)
		printf("  Press the ESCAPE key to %s.\n", escape);

	if ((msg) || (escape))
		printf("\n");
}



/*****************************************************************************
*
*	Function:	_error_status
*
*	Purpose:
*
*		Report that the API returned an error status.
*
*	Arguments:
*
*		prefix	A short message about the error.
*
*		status	The status returned.
*
*		arg		Any additional argument that may go woth the status.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _error_status(const char* prefix, U32 status, U32 arg)
{
	int	i;

	if (status != GSC_SUCCESS)
	{
		for (i = ARRAY_ELEMENTS(_err_list); i--;)
		{
			if (status == _err_list[i].status)
				break;
		}

		printf("ERROR: %s: 0x%lX", prefix, (long) status);

		if (i >= 0)
			printf(" (%s)", _err_list[i].text);

		printf(", 0x%lX\n", (long) arg);
	}
}



/*****************************************************************************
*
*	Function:	_verify_api
*
*	Purpose:
*
*		Verify that the API is usable.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _verify_api(void)
{
	U32	api_arg;
	U32	api_status;
	U8	count;
	U32	status;

	status	= hpdi32_api_status(&api_status, &api_arg, HPDI32_API_VERSION);

	if (status != GSC_SUCCESS)
	{
		_error_status("hpdi32_api_status()", status, 0);
	}
	else if (api_status != GSC_SUCCESS)
	{
		_error_status("API Status", api_status, api_arg);
		hpdi32_board_count(&count);
		printf("Boards Found: %d\n", (int) count);
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_verify_board_count
*
*	Purpose:
*
*		Make sure there are sufficient boards to access.
*
*	Arguments:
*
*		count		The number of boards installed goes here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _verify_board_count(U8* count)
{
	U32	status;

	status	= hpdi32_board_count(count);

	if (status != GSC_SUCCESS)
		_error_status("hpdi32_board_count()", status, 0);
	else if (count[0] == 0)
		printf("ERROR: no boards were found and two are required.\n");
	else if (count[0] == 1)
		printf("ERROR: only one board was found and two are required.\n");

	return(status);
}



/*****************************************************************************
*
*	Function:	_verify_board_selection
*
*	Purpose:
*
*		See which board indexes we're to use.
*
*	Arguments:
*
*		argc		The number of command line arguments.
*
*		argv		The list of command line arguments.
*
*		count		The number of boards installed.
*
*		board_1		The first board selection goes here.
*
*		board_2		The second board selection goes here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _verify_board_selection(
	int		argc,
	char**	argv,
	U8		count,
	int*	board_1,
	int*	board_2)
{
	U32	status	= GSC_SUCCESS;

	if (argc >= 2)
	{
		sscanf(argv[1], "%D", board_1);

		if ((board_1[0] < 0) || (board_1[0] >= count))
		{
			printf("ERROR: the first argument is invalid: %s\n", argv[1]);
			status	= GSC_FAILED;
		}
	}
	else
	{
		board_1[0]	= 0;
	}

	if ((status == GSC_SUCCESS) && (argc >= 3))
	{
		sscanf(argv[2], "%D", board_2);

		if ((board_2[0] < 0) ||
			(board_2[0] >= count) ||
			(board_2[0] == board_1[0]))
		{
			printf("ERROR: the second argument is invalid: %s\n", argv[2]);
			status	= GSC_FAILED;
		}
	}
	else if (board_1[0] == 0)
	{
		board_2[0]	= 1;
	}
	else
	{
		board_2[0]	= 0;
	}

	if ((status == GSC_SUCCESS) && (argc >= 4))
	{
		printf("ERROR: too many command line arguments.\n");
		status	= GSC_FAILED;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_access_boards
*
*	Purpose:
*
*		Gain access to the two boards.
*
*	Arguments:
*
*		board_1		The index of first board to access.
*
*		board_2		The index of second board to access.
*
*		handle_1	The handle of first board to access goes here.
*
*		handle_2	The handle of second board to access goes here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _access_boards(
	int		board_1,
	int		board_2,
	void**	handle_1,
	void**	handle_2)
{
	char	buf[64];
	U32		status;

	handle_1[0]	= NULL;
	handle_2[0]	= NULL;

	status	= hpdi32_open((U8) board_1, handle_1);

	if (status != GSC_SUCCESS)
	{
		sprintf(buf, "hpdi32_open(%d, &handle)", board_1);
		_error_status(buf, status, 0);
	}
	else
	{
		status	= hpdi32_open((U8) board_2, handle_2);

		if (status != GSC_SUCCESS)
		{
			sprintf(buf, "hpdi32_open(%d, &handle)", board_2);
			_error_status(buf, status, 0);
		}
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_get_board_type
*
*	Purpose:
*
*		Tell the caller the type of board we're accessing.
*
*	Arguments:
*
*		tx_index	The intended index for the transmit board.
*
*		tx_handle	The intended handle for the transmit board.
*
*		rx_index	The intended index for the receive board.
*
*		rx_handle	The intended handle for the receive board.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _get_board_type(void* handle, board_type_t* type)
{
	U32				didr	= 0;
	U32				frr		= 0;
	U32				sid		= 0;
	U32				stat;
	U32				status	= GSC_SUCCESS;
	U32				svid	= 0;
	U32				vidr	= 0;

	stat	= hpdi32_reg_read(handle, GSC_PCI_9080_VIDR, &vidr);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= hpdi32_reg_read(handle, GSC_PCI_9080_DIDR, &didr);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= hpdi32_reg_read(handle, GSC_PCI_9080_SVID, &svid);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= hpdi32_reg_read(handle, GSC_PCI_9080_SID, &sid);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= hpdi32_reg_read(handle, HPDI32_PRINT_FRR, &frr);
	status	= (status == GSC_SUCCESS) ? stat : status;

	if (status != GSC_SUCCESS)
	{
		type[0]	= BOARD_TYPE_OTHER;
	}
	else if (	(vidr == 0x10B5) &&
				(didr == 0x9080) &&
				(svid == 0x10B5) &&
				(sid  == 0x2400) &&
				(frr  == 0x00030306))
	{
		type[0]	= BOARD_TYPE_TX;
	}
	else if (	(vidr == 0x10B5) &&
				(didr == 0x9080) &&
				(svid == 0x10B5) &&
				(sid  == 0x2400) &&
				(frr  == 0x80010203))
	{
		type[0]	= BOARD_TYPE_RX;
	}
	else
	{
		type[0]	= BOARD_TYPE_OTHER;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_verify_board_types
*
*	Purpose:
*
*		Verify that the boards being accessed are the correct types.
*
*	Arguments:
*
*		tx_index	The intended index for the transmit board.
*
*		tx_handle	The intended handle for the transmit board.
*
*		rx_index	The intended index for the receive board.
*
*		rx_handle	The intended handle for the receive board.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _verify_board_types(
	int*	tx_index,
	void**	tx_handle,
	int*	rx_index,
	void**	rx_handle)
{
	void*			handle;
	int				index;
	const char*		psz;
	board_type_t	rx_type	= BOARD_TYPE_OTHER;
	U32				status;
	board_type_t	tx_type	= BOARD_TYPE_OTHER;

	status	= _get_board_type(tx_handle[0], &tx_type);

	if (status == GSC_SUCCESS)
		status	= _get_board_type(rx_handle[0], &rx_type);

	if ((tx_type == BOARD_TYPE_RX) && (rx_type == BOARD_TYPE_TX))
	{
		index			= rx_index[0];
		rx_index[0]		= tx_index[0];
		tx_index[0]		= index;
		handle			= rx_handle[0];
		rx_handle[0]	= tx_handle[0];
		tx_handle[0]	= handle;
		rx_type			= BOARD_TYPE_RX;
		tx_type			= BOARD_TYPE_TX;
	}

	if ((status == GSC_SUCCESS) &&
		((rx_type != BOARD_TYPE_RX) ||
		(tx_type != BOARD_TYPE_TX)))
	{
		status	= GSC_FAILED;
		printf("ERROR: the boards accessed are not the correct types.\n");

		switch (rx_type)
		{
			default:
			case BOARD_TYPE_OTHER:	psz	= "an unknown";	break;
			case BOARD_TYPE_RX:		psz	= "a TEST";		break;
			case BOARD_TYPE_TX:		psz	= "a PRINT";	break;
		}

		printf(	"  Board index %d is %s type board.\n",
				rx_index[0],
				psz);

		switch (tx_type)
		{
			default:
			case BOARD_TYPE_OTHER:	psz	= "an unknown";	break;
			case BOARD_TYPE_RX:		psz	= "a TEST";		break;
			case BOARD_TYPE_TX:		psz	= "a PRINT";	break;
		}

		printf(	"  Board index %d is %s type board.\n",
				tx_index[0],
				psz);
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_process_menu
*
*	Purpose:
*
*		Test access to the local/firmware registers.
*
*	Arguments:
*
*		list		The list of menu items to present.
*
*		size		The number of items in the above list.
*
*		tx_index	The intended index for the transmit board.
*
*		tx_handle	The intended handle for the transmit board.
*
*		rx_index	The intended index for the receive board.
*
*		rx_handle	The intended handle for the receive board.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _process_menu(
	const menu_item_t*	list,
	size_t				size,
	int					tx_index,
	void*				tx_handle,
	int					rx_index,
	void*				rx_handle)
{
	int		c;
	int		first	= 'A';
	int		last	= 'A' + (int) size - 1;
	size_t	s;

	for (;;)
	{
		_introductions(tx_index, rx_index, NULL, "exit");
		for (s = 0; s < size; s++)
			printf("    %c. %s\n", (char) 'A' + s, list[s].text);

		printf("\n");
		printf("  Make a selection from '%c' to '%c': ", (char) first, (char) last);
		c	= getch();

		if (c == 27)	// The escape key always works.
		{
			printf("\n");
			break;
		}

		printf("%c\n", (char) c);
		c	= toupper(c);

		if ((c >= first) && (c <= last))
		{
			c	-= 'A';

			if (list[c].function == NULL)
				break;

			(list[c].function)(tx_index, tx_handle, rx_index, rx_handle);
		}
		else
		{
			printf("\n  That was an invalid selection.\n");
		}

		printf("\n  Press a key to continue.");
		getch();
	}
}



/*****************************************************************************
*
*	Function:	_test_local_reg
*
*	Purpose:
*
*		Perform an access test of a firmware register.
*
*	Arguments:
*
*		handle		The handle for the device to access.
*
*		data		The data for the register to access.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _test_local_reg(void* handle, const reg_t* data)
{
	U32		errors	= 0;
	long	l;
	U32		original;
	U32		rd;
	U32		temp	= 0;
	U32		status	= GSC_SUCCESS;
	U32		wt		= 0;

	printf("    %-45s ", data->name);
	status	= hpdi32_reg_read(handle, data->reg, &original);

	if (status != GSC_SUCCESS)
	{
		_error_status("hpdi32_init()", status, 0);
	}
	else
	{
		for (l = 0; l < 0x1000; l++)
		{
			if (data->rw)
			{
				wt	= (original & ~data->mask)
					| ((0x55555555 << temp) & data->mask);
				status	= hpdi32_reg_write(handle, data->reg, wt);

				if (status != GSC_SUCCESS)
				{
					_error_status("hpdi32_reg_write()", status, 0);
					break;
				}
			}

			status	= hpdi32_reg_read(handle, data->reg, &rd);

			if (status != GSC_SUCCESS)
			{
				_error_status("hpdi32_reg_read()", status, 0);
				break;
			}

			if ((data->rw) && ((rd & data->mask) != (wt & data->mask)))
			{
				if(errors <= 10)
				{
					printf(	"\nERROR: Wrote %08lX, Read %08lX, loop --> %ld",
							(long) (wt & data->mask),
							(long) (rd & data->mask),
							l);
					status	= hpdi32_reg_read(handle, data->reg, &rd);
					printf("\n\t\tReread --> %08lX", rd & data->mask);
					errors++;
				}
			}

			temp++;
		}
	}

	if (!data->rw)
	{
	}
	else if (status == GSC_SUCCESS)
	{
		status	= hpdi32_reg_write(handle, data->reg, original);

		if (status != GSC_SUCCESS)
			_error_status("hpdi32_reg_write()", status, 0);
	}
	else
	{
		hpdi32_reg_write(handle, data->reg, original);
	}

	if ((status == GSC_SUCCESS) && (errors))
		status	= GSC_FAILED;

	if (status == GSC_SUCCESS)
		printf("PASS\n");

	return(status);
}



/*****************************************************************************
*
*	Function:	_test_local_regs
*
*	Purpose:
*
*		Test access to the local/firmware registers.
*
*	Arguments:
*
*		tx_index	The intended index for the transmit board.
*
*		tx_handle	The intended handle for the transmit board.
*
*		rx_index	The intended index for the receive board.
*
*		rx_handle	The intended handle for the receive board.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _test_local_regs(	int		tx_index,
								void*	tx_handle,
								int		rx_index,
								void*	rx_handle)
{
	static const reg_t	list[]	=
	{
		/* name									reg						rw	mask		*/
		{ "Firmware Revision Register",			HPDI32_PRINT_FRR,		0,	0x00000000	},
		{ "Board Control Register",				HPDI32_PRINT_BCR,		1,	0x00000010	},
		{ "Board Status Register",				HPDI32_PRINT_BSR,		0,	0x00000000	},
		{ "Tx Almost Register",					HPDI32_PRINT_TAR,		1,	0xFFFFFFFF	},
		{ "FIFO Data Register",					HPDI32_PRINT_FDR,		0,	0x00000000	},
		{ "Head Shift Size Register",			HPDI32_PRINT_HSSR,		1,	0x000000FF	},
		{ "Interrupt Control Register",			HPDI32_PRINT_ICR,		0,	0x00000000	},
		{ "Interrupt Status Register",			HPDI32_PRINT_ISR,		0,	0x00000000	},
		{ "Printing Control/Status Register",	HPDI32_PRINT_PCSR,		1,	0x0000FA3F	},
		{ "Edge Selector Register",				HPDI32_PRINT_ESR,		1,	0x001FFFFF	},
		{ "Head Data Holdoff Register",			HPDI32_PRINT_HDHR,		1,	0x00000FFF	},
		{ "Quadrature Transition Delay Count Register",
												HPDI32_PRINT_QTDCR,		1,	0x00000FFF	},
		{ "Start Position Register",			HPDI32_PRINT_STARTPR,	1,	0x000FFFFF	},
		{ "Stop Position Register",				HPDI32_PRINT_STOPPR,	1,	0x000FFFFF	},
		{ "Current Position Register",			HPDI32_PRINT_CPR,		0,	0x00000000	},
		{ "Head 0 Fire Delay Register",			HPDI32_PRINT_H0FDR,		1,	0x00000FFF	},
		{ "Head 0 Fire Width Register",			HPDI32_PRINT_H0FWR,		1,	0x00000FFF	},
		{ "Head 1 Fire Delay Register",			HPDI32_PRINT_H1FDR,		1,	0x00000FFF	},
		{ "Head 1 Fire Width Register",			HPDI32_PRINT_H1FWR,		1,	0x00000FFF	},
		{ "Head 2 Fire Delay Register",			HPDI32_PRINT_H2FDR,		1,	0x00000FFF	},
		{ "Head 2 Fire Width Register",			HPDI32_PRINT_H2FWR,		1,	0x00000FFF	},
		{ "Head 3 Fire Delay Register",			HPDI32_PRINT_H3FDR,		1,	0x00000FFF	},
		{ "Head 3 Fire Width Register",			HPDI32_PRINT_H3FWR,		1,	0x00000FFF	},
		{ "Head 4 Fire Delay Register",			HPDI32_PRINT_H4FDR,		1,	0x00000FFF	},
		{ "Head 4 Fire Width Register",			HPDI32_PRINT_H4FWR,		1,	0x00000FFF	},
		{ "Head 5 Fire Delay Register",			HPDI32_PRINT_H5FDR,		1,	0x00000FFF	},
		{ "Head 5 Fire Width Register",			HPDI32_PRINT_H5FWR,		1,	0x00000FFF	},
		{ "Head 6 Fire Delay Register",			HPDI32_PRINT_H6FDR,		1,	0x00000FFF	},
		{ "Head 6 Fire Width Register",			HPDI32_PRINT_H6FWR,		1,	0x00000FFF	},
		{ "Head 7 Fire Delay Register",			HPDI32_PRINT_H7FDR,		1,	0x00000FFF	},
		{ "Head 7 Fire Width Register",			HPDI32_PRINT_H7FWR,		1,	0x00000FFF	},
	};

	char	buf[256];
	int		c;
	U32		loops;
	size_t	s;
	U32		status;

	IGNORE_ARG(rx_index);
	IGNORE_ARG(rx_handle);

	for (loops = 1;; loops++)
	{
		if (kbhit())
		{
			c	= getch();

			if (c == 27)
				break;
		}

		sprintf(buf, "Local Register Access Tests (loop %lu)\n", (long) loops);
		_introductions(tx_index, -1, buf, "abort testing");
		status	= hpdi32_init(tx_handle);

		if (status != GSC_SUCCESS)
		{
			_error_status("hpdi32_init()", status, 0);
			break;
		}

		for (s = 0; s < ARRAY_ELEMENTS(list); s++)
		{
			status	= _test_local_reg(tx_handle, &list[s]);

			if (status != GSC_SUCCESS)
				break;
		}

		if (status != GSC_SUCCESS)
			break;
	}
}



/*****************************************************************************
*
*	Function:	_data_xfer_test
*
*	Purpose:
*
*		Perform a data transfer test from the Tx board to the Rx board.
*
*	Arguments:
*
*		tx_index	The intended index for the transmit board.
*
*		tx_handle	The intended handle for the transmit board.
*
*		rx_index	The intended index for the receive board.
*
*		rx_handle	The intended handle for the receive board.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _data_xfer_test(int		tx_index,
							void*	tx_handle,
							int		rx_index,
							void*	rx_handle)
{
	char	buf[256];
	long	bytes;
	int		c;
	int		errors	= 0;
	float	f;
	long	k;
	U32		loops;
	int		remainder;
	U32		rx;
	U32		rx_bsr;
	U32		rx_buf[32L * 1024];
	U32		rx_status;
	size_t	s;
	U32		stat;
	U32		status	= GSC_SUCCESS;
	U32		tx;
	U32		tx_bsr;
	U32		tx_buf[32L * 1024];
	U32		tx_status;

	memset(rx_buf, 0, sizeof(rx_buf));
	memset(tx_buf, 0, sizeof(tx_buf));

	/*
	*	Configure the PRINT board.
	*/

	stat	= hpdi32_init(tx_handle);
	status	= (status == GSC_SUCCESS) ? stat : status;

	stat	= hpdi32_reg_write(tx_handle, HPDI32_PRINT_HSSR, 0x80);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= hpdi32_reg_write(tx_handle, HPDI32_PRINT_PCSR, 0xF0030);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= hpdi32_reg_write(tx_handle, HPDI32_PRINT_ESR, 0x4811);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= hpdi32_reg_write(tx_handle, HPDI32_PRINT_QTDCR, 0x00);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= hpdi32_reg_write(tx_handle, HPDI32_PRINT_HDHR, 0x10);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= hpdi32_reg_write(tx_handle, HPDI32_PRINT_STARTPR, 0x10);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= hpdi32_reg_write(tx_handle, HPDI32_PRINT_STOPPR, 0x100);
	status	= (status == GSC_SUCCESS) ? stat : status;

	stat	= hpdi32_reg_write(tx_handle, HPDI32_PRINT_H0FDR, 0x1);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= hpdi32_reg_write(tx_handle, HPDI32_PRINT_H1FDR, 0x2);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= hpdi32_reg_write(tx_handle, HPDI32_PRINT_H2FDR, 0x3);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= hpdi32_reg_write(tx_handle, HPDI32_PRINT_H3FDR, 0x4);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= hpdi32_reg_write(tx_handle, HPDI32_PRINT_H4FDR, 0x5);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= hpdi32_reg_write(tx_handle, HPDI32_PRINT_H5FDR, 0x6);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= hpdi32_reg_write(tx_handle, HPDI32_PRINT_H6FDR, 0x7);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= hpdi32_reg_write(tx_handle, HPDI32_PRINT_H7FDR, 0x8);
	status	= (status == GSC_SUCCESS) ? stat : status;

	stat	= hpdi32_reg_write(tx_handle, HPDI32_PRINT_H0FWR, 0x10);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= hpdi32_reg_write(tx_handle, HPDI32_PRINT_H1FWR, 0x10);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= hpdi32_reg_write(tx_handle, HPDI32_PRINT_H2FWR, 0x10);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= hpdi32_reg_write(tx_handle, HPDI32_PRINT_H3FWR, 0x10);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= hpdi32_reg_write(tx_handle, HPDI32_PRINT_H4FWR, 0x10);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= hpdi32_reg_write(tx_handle, HPDI32_PRINT_H5FWR, 0x10);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= hpdi32_reg_write(tx_handle, HPDI32_PRINT_H6FWR, 0x10);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= hpdi32_reg_write(tx_handle, HPDI32_PRINT_H7FWR, 0x10);
	status	= (status == GSC_SUCCESS) ? stat : status;

	stat	= HPDI32_IO_MODE__TX_DMA(tx_handle);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= HPDI32_IO_DMA_CONTROL_MODE__TX_MANUAL(tx_handle);
	status	= (status == GSC_SUCCESS) ? stat : status;

	stat	= HPDI32_FIFO_ALMOST_LEVEL__TX_AE_SET(tx_handle, 0x10);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= HPDI32_FIFO_ALMOST_LEVEL__TX_AF_SET(tx_handle, 0x10);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= HPDI32_TX_AUTO_START__YES(tx_handle);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= HPDI32_TX_ENABLE__YES(tx_handle);
	status	= (status == GSC_SUCCESS) ? stat : status;

	/*
	*	Configure the TEST board.
	*/

	stat	= hpdi32_init(rx_handle);
	status	= (status == GSC_SUCCESS) ? stat : status;

	stat	= hpdi32_reg_write(rx_handle, HPDI32_PRINT_TEST_BOARD_QTDCR, 0x10);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= hpdi32_reg_write(rx_handle, HPDI32_PRINT_TEST_BOARD_PCSR, 0x7);
	status	= (status == GSC_SUCCESS) ? stat : status;

	stat	= HPDI32_IO_MODE__RX_DMA(rx_handle);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= HPDI32_IO_DMA_CONTROL_MODE__RX_MANUAL(rx_handle);
	status	= (status == GSC_SUCCESS) ? stat : status;

	stat	= HPDI32_FIFO_ALMOST_LEVEL__RX_AE_SET(rx_handle, 0x10);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= HPDI32_FIFO_ALMOST_LEVEL__RX_AF_SET(rx_handle, 0x10);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= HPDI32_RX_ENABLE__YES(rx_handle);
	status	= (status == GSC_SUCCESS) ? stat : status;

	/*
	*	Perform the data transfer test.
	*/

	for (loops = 1; status == GSC_SUCCESS; loops++)
	{
		if (kbhit())
		{
			c	= getch();

			if (c == 27)
				break;
		}

		sprintf(buf, "Data Transfer Test (loop %lu)\n", (long) loops);
		_introductions(tx_index, rx_index, buf, "abort testing");

		// Initialize the Tx buffer.

		for (s = 0; s < ARRAY_ELEMENTS(tx_buf); s++)
		{
			remainder	= s % 5;
			
			switch(remainder)
			{
				default:
				case 0:	tx_buf[s] = 0xAAAAAAAA;	break;
				case 1:	tx_buf[s] = 0x55555555;	break;
				case 2:	tx_buf[s] = 0x00000000;	break;
				case 3:	tx_buf[s] = 0xFFFFFFFF;	break;
				case 4:	tx_buf[s] = s / 5;		break;
			}
		}

		// Initialize the Rx buffer.

		for (s = ARRAY_ELEMENTS(rx_buf); s--;)
			rx_buf[s]	= 0xDEADBEEF;

		// Write the data.
		bytes	= sizeof(tx_buf);
		k		= bytes / 4 / 1024;
		printf("Writing %lu bytes (%luK samples) ... ", bytes, k);
		fflush(stdout);
		tx	= 0;
		status	= hpdi32_write(tx_handle, tx_buf, (U32) bytes, &tx);
		printf("done (%lu)\n", (long) tx);
		f	= ((float) (loops - 1) * bytes) + tx;
		_format_float(f, buf);
		printf("Total Tx bytes: %s\n", buf);

		if (status != GSC_SUCCESS)
		{
			printf("ERROR: hpdi32_write(): 0x%lX\n", (long) status);
			stat	= hpdi32_reg_read(tx_handle, HPDI32_PRINT_BSR, &tx_bsr);
			status	= (status == GSC_SUCCESS) ? stat : status;
			printf("Tx Board Status Register: 0x%08lX\n", (long) tx_bsr);
			break;
		}

		// Wait for data transmission to complete.
		printf("Waiting for Tx FIFO to empty ... ");

		for (s = 500000UL;;s--)
		{
			if (s == 0)
			{
				status	= GSC_FAILED;
				printf("\nERROR: Tx FIFO didn't empty.\n");
				break;
			}

			status	= HPDI32_FIFO_STATUS__TX_GET(tx_handle, &tx_status);

			if (status != GSC_SUCCESS)
			{
				printf(	"\nERROR: HPDI32_FIFO_STATUS__TX_GET(): 0x%lX\n",
						(long) status);
				break;
			}

			if (tx_status == HPDI32_FIFO_STATUS_EMPTY)
				break;
		}

		if (status != GSC_SUCCESS)
			break;

		printf("done\n");

		// Wait for the data transmission to complete.
		printf("Waiting for Rx FIFO to fill ... ");

		for (s = 500000UL;;s--)
		{
			if (s == 0)
			{
				status	= GSC_FAILED;
				printf("\nERROR: Rx FIFO didn't fill.\n");
				break;
			}

			status	= HPDI32_FIFO_STATUS__RX_GET(rx_handle, &rx_status);

			if (status != GSC_SUCCESS)
			{
				printf(	"\nERROR: HPDI32_FIFO_STATUS__RX_GET(): 0x%lX\n",
						(long) status);
				break;
			}

			if (rx_status == HPDI32_FIFO_STATUS_FULL)
				break;
		}

		if (status != GSC_SUCCESS)
			break;

		printf("done\n");

		// Read the data.
		bytes	= sizeof(rx_buf);
		k		= bytes / 4 / 1024;
		printf("Reading %lu bytes (%luK samples) ... ", bytes, k);
		fflush(stdout);
		rx	= 0;
		status	= hpdi32_read(rx_handle, rx_buf, (U32) bytes, &rx);
		printf("done (%lu)\n", (long) rx);
		f	= ((float) (loops - 1) * bytes) + rx;
		_format_float(f, buf);
		printf("Total Rx bytes: %s\n", buf);

		if (status != GSC_SUCCESS)
		{
			printf("ERROR: hpdi32_read(): 0x%lX\n", (long) status);
			stat	= hpdi32_reg_read(rx_handle, HPDI32_PRINT_BSR, &rx_bsr);
			status	= (status == GSC_SUCCESS) ? stat : status;
			printf("Rx Board Status Register: 0x%08lX\n", (long) rx_bsr);
			break;
		}

		// Validate the received data.
		printf("Validating data ... ");

		for (s = 0; s < (ARRAY_ELEMENTS(rx_buf) - 1); s++)
		{
			if ((rx_buf[s + 1] & 0xFFFF) != (tx_buf[s] & 0xFFFF))
			{
				errors++;

				if (errors <= 1)
					printf("FAIL\n");

				if (errors >= 100)
					break;

				if (errors > 20)
					continue;

				printf(	"ERROR: word %lu: read 0x%04lX, expected 0x%04lX\n",
						(long) s,
						(long) (rx_buf[s + 1] & 0xFFFF),
						(long) (tx_buf[s] & 0xFFFF));
			}
		}

		if (errors >= 100)
			break;
		else
			printf("pass\n");
	}

	/*
	*	Reset things.
	*/

	stat	= hpdi32_init(tx_handle);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= hpdi32_init(rx_handle);
	status	= (status == GSC_SUCCESS) ? stat : status;
}



/*****************************************************************************
*
*	Function:	main
*
*	Purpose:
*
*		Implement the application's entry point.
*
*	Arguments:
*
*		argc	The number of command line arguments.
*
*		argv	The list of command line arguments.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void main(int argc, char** argv)
{
	static const menu_item_t	list[]	=
	{
		/* text									function			*/
		{ "Test Local Print Board Registers",	_test_local_regs	},
		{ "Data Transfer Test",					_data_xfer_test		},
		{ "Exit to system",						NULL				}
	};

	U8		count		= 0;
	void*	rx_handle	= NULL;
	int		rx_index	= 0;
	U32		status;
	void*	tx_handle	= NULL;
	int		tx_index	= 0;

	status	= _verify_api();

	if (status == GSC_SUCCESS)
		status	= _verify_board_count(&count);

	if (status == GSC_SUCCESS)
		status	= _verify_board_selection(argc, argv, count, &rx_index, &tx_index);

	if (status == GSC_SUCCESS)
		status	= _access_boards(rx_index, tx_index, &rx_handle, &tx_handle);

	if (status == GSC_SUCCESS)
		status	= _verify_board_types(&tx_index, &tx_handle, &rx_index, &rx_handle);

	if (status == GSC_SUCCESS)
	{
		_process_menu(	list,
						ARRAY_ELEMENTS(list),
						tx_index,
						tx_handle,
						rx_index,
						rx_handle);
	}

	hpdi32_close(tx_handle);
	hpdi32_close(rx_handle);
}


